@@ -31,7 +31,7 @@
 # - \fB switches to boldface
 # - \fC swtiches to a literal string
 # - \fP switches to normal display
-# - .dl on a line starts a definition list (name taken taken from HTML).
+# - .dl on a line starts a definition list (name taken from HTML).
 # - .dt starts a term in a definition list.
 # - .dd starts a definition in a definition list.
 # - .de on a line finishes a definition list.
@@ -96,24 +96,28 @@ my $SP_END_SECT    = 18;
 my $SP_REFER       = 19;
 
 # Types to documentation readable strings:
-my %type2human = ("DT_NONE"  => "-none-",
-                  "DT_BOOL"  => "boolean",
-                  "DT_NUM"   => "number",
-                  "DT_LNUM"  => "number (long)",
-                  "DT_STR"   => "string",
-                  "DT_PATH"  => "path",
-                  "DT_QUAD"  => "quadoption",
-                  "DT_SORT"  => "sort order",
-                  "DT_RX"    => "regular expression",
-                  "DT_MAGIC" => "folder magic",
-                  "DT_ADDR"  => "e-mail address",
-                  "DT_MBCHARTBL"=> "string");
+my %type2human = ("DT_NONE"      => "-none-",
+                  "DT_BOOL"      => "boolean",
+                  "DT_NUM"       => "number",
+                  "DT_LNUM"      => "number (long)",
+                  "DT_STR"       => "string",
+                  "DT_PATH"      => "path",
+                  "DT_CMD_PATH"  => "path",
+                  "DT_QUAD"      => "quadoption",
+                  "DT_SORT"      => "sort order",
+                  "DT_RX"        => "regular expression",
+                  "DT_MAGIC"     => "folder magic",
+                  "DT_ADDR"      => "e-mail address",
+                  "DT_MBCHARTBL" => "string",
+                  "DT_L10N_STR"  => "string (localized)");
 
-my %string_types = ("DT_STR"  => 1,
-                    "DT_RX"   => 1,
-                    "DT_ADDR" => 1,
-                    "DT_PATH" => 1,
-                    "DT_MBCHARTBL" => 1);
+my %string_types = ("DT_STR"       => 1,
+                    "DT_RX"        => 1,
+                    "DT_ADDR"      => 1,
+                    "DT_PATH"      => 1,
+                    "DT_CMD_PATH"  => 1,
+                    "DT_MBCHARTBL" => 1,
+                    "DT_L10N_STR"  => 1);
 
 my %quad2human = ("MUTT_YES" => "yes",
                   "MUTT_NO"  => "no",
@@ -239,13 +243,25 @@ sub flush_doc($) {
 sub handle_confline($) {
   my ($line) = @_;
 
+  my $localized = 0;
   my ($name, $type, $flags, $data, $val) = split(/\s*,\s*/, $line, 5);
   $name =~ s/"//g;
 
-  $type =~ s/\|.*//;
+  if ($type =~ /DT_L10N_STR/) {
+    $localized = 1;
+    $type = "DT_L10N_STR";
+  }
+  else {
+    $type =~ s/\|.*//;
+  }
 
   $val =~ s/^{\s*\.[lp]\s*=\s*"?//;
   $val =~ s/"?\s*}\s*},\s*$//;
+  if ($localized) {
+    $val =~ s/^N_\s*\("//;
+    $val =~ s/"\)$//;
+  }
+
   # This is a hack to concatenate compile-time constants.
   # (?<!..) is a zero-width negative lookbehind assertion, asserting
   # the first quote isn't preceded by a backslash
@@ -741,7 +757,7 @@ sub print_docline_man($$$$) {
     $docstat &= ~($D_BF|$D_TT);
   }
   elsif ($special == $SP_START_TT) {
-    print "\\fC";
+    print "\\fB";
     $docstat |= $D_TT;
     $docstat &= ~($D_BF|$D_EM);
   }
