@@ -1260,7 +1260,7 @@ hcache_open_db4 (struct header_cache* h, const char* path)
 {
   struct stat sb;
   int ret;
-  u_int32_t createflags = DB_CREATE;
+  uint32_t createflags = DB_CREATE;
   int pagesize;
 
   pagesize = HeaderCachePageSize;
