@@ -141,6 +141,12 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
   char		qc = 0; /* quote char */
   char		*pc;
 
+  /* Some callers used to rely on the (bad) assumption that dest->data
+   * would be non-NULL after calling this function.  Perhaps I've missed
+   * a few cases, or a future caller might make the same mistake.
+   */
+  if (!dest->data)
+    mutt_buffer_increase_size (dest, STRING);
   mutt_buffer_clear (dest);
 
   SKIPWS (tok->dptr);
@@ -3734,13 +3740,15 @@ void mutt_init (int skip_sys_rc, LIST *commands)
 
   if ((p = getenv ("REPLYTO")) != NULL)
   {
-    BUFFER token;
+    BUFFER *token;
     union pointer_long_t udata = {.l=0};
 
     mutt_buffer_printf (buffer, "Reply-To: %s", p);
-    mutt_buffer_init (&token);
-    parse_my_hdr (&token, buffer, udata, &err);
-    FREE (&token.data);
+    buffer->dptr = buffer->data;
+
+    token = mutt_buffer_pool_get ();
+    parse_my_hdr (token, buffer, udata, &err);
+    mutt_buffer_pool_release (&token);
   }
 
   if ((p = getenv ("EMAIL")) != NULL)
