@@ -41,7 +41,8 @@ enum
   MUTT_NEW_MAIL = 1,    /* new mail received in mailbox */
   MUTT_LOCKED,          /* couldn't lock the mailbox */
   MUTT_REOPENED,        /* mailbox was reopened */
-  MUTT_FLAGS            /* nondestructive flags change (IMAP) */
+  MUTT_FLAGS,           /* nondestructive flags change (IMAP) */
+  MUTT_RECONNECTED      /* lost connection. reconnected (IMAP) */
 };
 
 typedef struct _message
