@@ -56,6 +56,11 @@ int mutt_tunnel_socket_setup (CONNECTION *conn)
   conn->conn_write = tunnel_socket_write;
   conn->conn_poll = tunnel_socket_poll;
 
+  /* Note we are using ssf as a boolean in this case.  See the notes
+   * in mutt_socket.h */
+  if (option (OPTTUNNELISSECURE))
+    conn->ssf = 1;
+
   return 0;
 }
 
