@@ -19,7 +19,7 @@
 #define SORT_DATE	1   /* the date the mail was sent. */
 #define SORT_SIZE	2
 #define SORT_SUBJECT	3
-#define SORT_ALPHA	3   /* makedoc.c requires this */
+#define SORT_ALPHA	3   /* makedoc.pl requires this */
 #define SORT_FROM	4
 #define SORT_ORDER	5   /* the order the messages appear in the mailbox. */
 #define SORT_THREADS	6
